# Copyright 2009-2010 Ram Rachum. No part of this program may be used, copied
# or distributed without explicit written permission from Ram Rachum.

'''
A package subclassing classes from PyAUI.

This is in order to have an AUI that look and functions the way we like it.
'''

from aui_manager import AuiManager
from aui_dock_art import AuiDockArt
from aui_tab_art import AuiTabArt