# Copyright 2009-2010 Ram Rachum. No part of this program may be used, copied
# or distributed without explicit written permission from Ram Rachum.

'''
This package defines generic wxPython widgets.
'''

from knob import Knob
from cute_dialog import CuteDialog