# Copyright 2009-2010 Ram Rachum. No part of this program may be used, copied
# or distributed without explicit written permission from Ram Rachum.

'''
This package defines the TreeBrowser class.

See its documentation in the `tree_browser.py` module for more information.
'''

from tree_browser import TreeBrowser

__all__ = ["TreeBrowser"]