# Copyright 2009-2010 Ram Rachum. No part of this program may be used, copied
# or distributed without explicit written permission from Ram Rachum.

'''
Defines the PlaybackControls class.

See its documentation for more info.
'''

from playback_controls import PlaybackControls

__all__ = ["PlaybackControls"]