# Copyright 2009-2010 Ram Rachum. No part of this program may be used, copied
# or distributed without explicit written permission from Ram Rachum.

'''
This is a bootstrap package for garlicsim_wx. See module bootstrap here.
'''

from . import bootstrap