# Copyright 2009-2010 Ram Rachum. No part of this program may be used, copied
# or distributed without explicit written permission from Ram Rachum.

'''
This package defines a variety of little objects that are completely decoupled
from garlicsim_wx and do not belong anywhere else.
'''