# Copyright 2009-2010 Ram Rachum. No part of this program may be used, copied
# or distributed without explicit written permission from Ram Rachum.

'''
An emitter mechanism, a variation on the publisher-subscriber design pattern.
'''

from emitter import Emitter
from emitter_system import EmitterSystem